import java.awt.*;

/**
 * Class that represents a frog, which is "born" as an egg.  
 * Frogs can also be either tadpoles of full grown frogs.
 * See descriptions below for methods.
 * 
 * Problem 1.a:   (5 pts) The States of a Frog:
 * From the above description and test code, how many states 
 * do your think a Frog has?   Describe the states, in particular, 
 * give examples of how they differ from each other. 
 * STUDENT TO COMPLETE:
 * 
 * A Frog has 4 states, egg, tadpole, adult and dead.
 * eggs grow into tadpoles which grow into adults.
 * eggs and tadpoles and dead frogs are ok underwater.
 * adult and dead frogs are ok above water.
 * All states have different colors.
 */
public class Frog {
  
  //Problem 1.c.(5 pts) Figure out what methods the nested abstract state class, 
  //AState, must have order for it to properly represent the union of 
  //all the concrete states and write their declarations.   
  //Minimize the number of input parameters to these methods!
  
  // STUDENT TO COMPLETE, 
  // ADDING ANY NECESSARY CLASSES AND/OR FIELDS AND/OR METHODS.
  //
  // THE STATE DESIGN PATTERN MUST BE IMPLEMENTED USING 
  // ANONYMOUS INNER CLASSES!
  
  /**
   * Represents the abstract state of the Frog.
   */
  private static abstract class AState {
    /**
     * The state-dependent behavior when you wait 1 month.
     */
    abstract void wait1Month();
    
    /**
     * The state-dependent color
     */
    abstract Color getColor();
    
    /**
     * The state-dependent behavior when you hold the Frog above water for a long time
     */
    abstract void holdAboveWater();
    
    /**
     * The state-dependent behavior when you hold the Frog below water for a long time
     */
    abstract void holdBelowWater();
  }
  
  /**
   * State of being an egg
   */
  private AState eggState = new AState() {
    /**
     * Hatches into a tadpole
     */
    void wait1Month() {
      state = tadpoleState;
    }
    
    /**
     * Eggs are pink
     */
    Color getColor() {
      return Color.PINK;
    }
    
    /**
     * Eggs die in the air
     */
    void holdAboveWater() {
      state = deadState;
    }
    
    /**
     * Eggs are ok below the water
     */
    void holdBelowWater() {
    }
  };
  
  /**
   * The tadpole state
   */
  private AState tadpoleState = new AState() {
    
    /**
     * Grows into an adult frog
     */
    void wait1Month() {
      state = adultState;
    }
    
    /**
     * Tadpoles are gray
     */
    Color getColor() {
      return Color.GRAY;
    }
    
    /**
     * Tadpoles die above the water
     */
    void holdAboveWater() {
      state = deadState;
    }
    
    /**
     * Tadpoles are ok below the water
     */
    void holdBelowWater() {
    }
  };
  
  private AState adultState = new AState() {
    /**
     * Adults live for a long time
     */
    void wait1Month() {
    }
    
    /**
     * Adults are green
     */
    Color getColor() {
      return Color.GREEN;
    }
    
    /**
     * Adults are ok above the water.
     */
    void holdAboveWater() {
    }
    
    /**
     * Adults drown underwater
     */
    void holdBelowWater() {
      state = deadState;
    }
  };
  
  /**
   * The dead frog state
   */
  private AState deadState = new AState() {
    
    /**
     * Dead stays dead
     */
    void wait1Month() {
    }
    
    /**
     * Dead frogs are black
     */
    Color getColor() {
      return Color.BLACK;
    }
    
    /**
     * Doesn't matter where a dead frog is.
     */
    void holdAboveWater() {
    }
    
    /**
     * Doesn't matter where a dead frog is.
     */
    void holdBelowWater() {
    }
  };
  
  /**
   * The current state of the Frog, initially an egg
   */
  private AState state = eggState;
  
  
  /**
   * Wait one month.  The frog's state will change appropriately.
   * 
   * (Problem 1.b 5 pts) STUDENT TO COMPLETE: Describe this method's 
   * behavior for all possible states of the Frog.
   * 
   * egg -> tadpole -> adult
   * dead -> no change
   */
  public void wait1Month() {
    // STUDENT TO COMPLETE
    state.wait1Month();
  }
  
  /**
   * Returns the current color of the frog.
   * Depends on what state the frog is in.
   * 
   * (Problem 1.b 5 pts) STUDENT TO COMPLETE: Describe this method's 
   * behavior for all possible states of the Frog.
   * 
   * egg = pink, tadpole = gray, adult = green, dead = black
   * 
   */
  public Color getColor() {
    // STUDENT TO COMPLETE
    return state.getColor();
  }
  
  /**
   * Hold the frog above the water (in the air) for an 
   * extended period of time.  The effect on 
   * the frog will depend on its state.
   * 
   * (Problem 1.b 5 pts) STUDENT TO COMPLETE: Describe this method's 
   * behavior for all possible states of the Frog.
   * 
   * egg, tadpole -> dead
   * adult, dead -> no change
   */
  public void holdAboveWater() {
    // STUDENT TO COMPLETE
    state.holdAboveWater();
  }
  
  
  /**
   * Hold the frog under the water for an 
   * extended period of time.  The effect on 
   * the frog will depend on its state.
   * 
   * (Problem 1.b 5 pts) STUDENT TO COMPLETE: Describe this method's 
   * behavior for all possible states of the Frog.
   * 
   * egg, tadpole, dead -> no change
   * adult -> dead
   * 
   */
  public void holdBelowWater() {
    // STUDENT TO COMPLETE
    state.holdBelowWater();
  }
  
}