package genList;
import listFW.*;

/**
 * A concrete implementation of INodeFactory that creates 
 * concrete implementations of IDataNode and IListNode.
 * All implementation classes are completely hidden internally.
 */
public class NodeFactory implements INodeFactory {
  /**
   * Singleton design pattern
   */
  public static final NodeFactory Singleton = new NodeFactory();
  
  private NodeFactory() {}
  
  /**
   * Instantiates a new IDataNode instance that holds the given
   * data.
   * The String representation of this instance is the String 
   * representation of the data surrounded by square brackets:
   * "[data.toString()]"
   * @param data The data this node holds.
   * @return An IDataNode instance
   */
  public IDataNode makeDataNode(final Object data) {
    return new IDataNode() {
      
      public Object getData() {
        return data;
      }
      
      public Object execute(INodeAlgo algo, Object... inp) {
        return algo.dataCase(this, inp);
      }    
      
      public String toString() {
        return "["+data+"]";
      }
    };
  }
  
  /**
   * Instantiates a new IListNode instance that holds the given
   * IList.
   * The String representation of this instance is the String 
   * representation of the d surrounded by square brackets:
   * "[list.toString()]"
   * @param list The IList this node holds.
   * @return An IListNode instance
   */
  public IListNode makeListNode(final IList list) {
    return new IListNode() {
      
      public IList getList() {
        return list;
      }
      
      public Object execute(INodeAlgo algo, Object... inp) {
        return algo.listCase(this, inp);
      }
      
      public String toString() {
        return "["+list+"]";
      }
    };
  }
}