package genList.test;
import genList.*;
import listFW.*;
import listFW.factory.*;
import listFW.visitor.*;
import junit.framework.TestCase;


public class GenList_Test extends TestCase {
  
  public void test_CountGenList() {
    IListAlgo algo = CountGenList.Singleton;
    IListFactory lFac = InnerCompListFactory.Singleton;
    INodeFactory nFac = NodeFactory.Singleton;
    
    IList gl = lFac.makeEmptyList();    
    // gl is an empty list
    assertEquals("empty list", 0, gl.execute(algo));
    
    gl  = lFac.makeNEList(nFac.makeDataNode("a"), gl);
    // gl is a one element list whose first holds a data element.
    assertEquals("([a])","([a])", gl.toString());
    assertEquals("([a])", 1, gl.execute(algo));
    
    gl  = lFac.makeNEList(nFac.makeListNode(lFac.makeEmptyList()), lFac.makeEmptyList());
    // gl is a one element list whose first holds an empty list.
    assertEquals("([()])","([()])", gl.toString());
    assertEquals("([()])", 0, gl.execute(algo));
    
    gl  = lFac.makeNEList(nFac.makeListNode(lFac.makeNEList(nFac.makeDataNode("b"),
                                                            lFac.makeEmptyList())), 
                          lFac.makeEmptyList());
    // gl is not a one element list whose first holds a one element list.
    assertEquals("([([b])])","([([b])])", gl.toString());
    assertEquals("([([b])])", 1, gl.execute(algo));
    
    
    gl  = lFac.makeNEList(nFac.makeListNode(lFac.makeNEList(nFac.makeDataNode("x"),
                                                            lFac.makeNEList(nFac.makeDataNode("y"),
                                                                            lFac.makeEmptyList()))), 
                          gl);
    // gl is now a two element list, where the first element holds 
    // a two element list and whose second element holds a one 
    // element list.
    assertEquals("([([x], [y])], [([b])])","([([x], [y])], [([b])])", gl.toString());
    assertEquals("([([x], [y])], [([b])])", 3, gl.execute(algo));
  }
  
}
