package lrs.visitor;
import lrs.*;
import java.util.*;

/**
 * Sorts a list of Objects using a modified selection sort 
 * algorithm and a Comparator, given as the input parameter.
 * Mutates the host list to be sorted.
 * Returns the mutated host list.
 */
public class SelectSort implements IAlgo {
  
  public static final SelectSort Singleton = new SelectSort();
  private SelectSort() {}
  
  /**
   * Host is empty.  
   * Returns the mutated host.
   * @param host The empty host list
   * @param inp Not used.
   * @return The host list
   */
  public Object emptyCase(LRStruct host, Object... inp) {
    // STUDENT TO COMPLETE
    return host;
  }
  
  /**
   * List is non-empty.  Forward accumulates the *location* of the 
   * minimum value in the list, as determined by the given Comparator.  Removes the minimum 
   * value from the list and then sorts the remaining elements.
   * After that is done, insert the removed minimum value at the 
   * front of the host list.
   * @param host The non-empty host list to be mutated.
   * @param inp inp[0] is a Comparator object to be used to compare values.
   * @return The mutated host list.
   */
  public Object nonEmptyCase(final LRStruct host, Object... inp) {
    final Comparator comp = (Comparator)inp[0];
    // Forward accumulate the location of the minimum value in the list
    return ((LRStruct)host.getRest().execute(new IAlgo() {
      /**
       * The end of the list.  The location of the minimum value 
       * has been found.  Remove the minimum element, sort the remaining
       * elements of the list and insert the minimum element at the
       * front of the list.  
       * @param h The current host list 
       * @param min  min[0] is an LRStruct which is the accumulated location of the minimum value  
       * @return The mutated original host list.
       */
      public Object emptyCase(LRStruct h, Object... min) {
        // Problem 3.b (10 pts) STUDENT TO COMPLETE
        Object x = ((LRStruct)min[0]).removeFront();  // remove the min element from the entire list
        host.execute(SelectSort.this, comp);  // sort the entire list
        return host.insertFront(x); // put min element back onto the front of the sorted list.
      }
      
      /**
       * Not at end of list yet, continue accumulating the location
       * of the minimum value.
       * Note:  comp.compare(x, y) will be less than zero for x<y.
       * @param h The current host list 
       * @param min  min[0] is an LRStruct which is the accumulated location of the minimum value  
       * @return The mutated original host list.
       */
      public Object nonEmptyCase(LRStruct h, Object... min) {
        // Problem 3.a (5 pts) STUDENT TO COMPLETE
        
        // Recur with the location of the min value
        return h.getRest().execute(this, (0 > comp.compare(h.getFirst(), ((LRStruct)min[0]).getFirst())) ? h : min[0]);
      }
    }, host));  // STUDENT TO PROVIDE CORRECT INPUT PARAMETER
  }
}