package lrs.visitor;
import lrs.*;
import java.util.*;
import ring.*;

/**
 * Sorts a list of Objects using a modified selection sort 
 * algorithm and a Comparator, given as the input parameter.
 * Mutates the host list to be sorted.
 * Returns the mutated host list.
 */
public class SmoothingAlgo implements IAlgo {
  
  /**
   * Singleton Design Pattern
   */
  public static final SmoothingAlgo Singleton = new SmoothingAlgo();
  private SmoothingAlgo() {}
  
  
  /**
   * Host is empty.  
   * Returns the mutated host.
   * @param host The empty host list
   * @param winSize Not used.
   * @return The host list
   */
  public Object emptyCase(LRStruct host, Object... winSize) {
    // 4.a (5 pts) STUDENT TO COMPLETE
    return host;
  }
  
  /**
   * Non-empty list.  Must set up ring buffer to hold original values 
   * in smoothing window.
   * @param winSize winSize[0] == the number of elements to include in the smoothing window
   * @return The mutated host.
   */
  public Object nonEmptyCase(final LRStruct host, Object... winSize) {
    final RingBuffer ring = (new RingBuffer()).insertFront(host.getFirst());
    // At this stage, the ring currently sits 
    // at the newest element entered.
    // The "previous" direction points backwards, 
    // from the newest element directly at the oldest element entered.
    
    return ((LRStruct)host.getRest().execute(new IAlgo() {
      
      /**
       * End of list encountered.  Done with smoothing process.
       * return mutated host.
       */
      public Object emptyCase(LRStruct h, Object... inp) {
        // 4.b (5 pts) STUDENT TO COMPLETE
    
        return host;
      }
      
      /**
       * More data elements to process.
       * Need to use the previous total sum and ring data to calculate the current total sum 
       * and hence, current average value.
       * A positive countDown value means that the sliding window is not full yet, so the average 
       * needs to be calculated differently.
       * @param inp inp[0] = the current window size countdown value.  inp[1] = previous total sum.
       * @return The mutated host.
       */
      public Object nonEmptyCase(LRStruct h, Object... inp) {
        Integer countDown = (Integer)inp[0];
        Double prevSum = (Double) inp[1];
        // Here, the ring should sit at the newest element entered.
        // countDown will be zero if the window is full, >0 otherwise.
        // prevSum is the total sum of the ring.getCount() elements before this element.
        // 4.c (15 pts) STUDENT TO COMPLETE
        
        // Begin to build the current sum by adding the current raw value 
        Double curSum = prevSum + (Double)h.getFirst(); 
        ring.insertFront(h.getFirst()); // add current value to ring
        
        // Different process if the max averaging window already encountered.
        if(0<countDown) {
          // Window is not full, don't want to pull any data out of the ring yet.
          countDown--; // decrement the countdown value.
        }
        else {
          // At max window size, i=0 from now on.
          curSum -= (Double)ring.toPrev().removeFront();  // Remove oldest value from total sum 
          // Ring now points to newest value
        }

        // Calculate the average and save it.
        h.setFirst(curSum/ring.getCount());
        
        return h.getRest().execute(this, countDown, curSum); // recur with the current countdown value and the current total sum.
      }
    }, ((Integer) winSize[0])-1, host.getFirst())); // winSize is decremented b/c we have one element now.  Host's first is also the total sum for one element.
  }
}