package ring.test;
import ring.*;

import junit.framework.TestCase;


public class RingBuffer_Test extends TestCase {
  

  public void test_empty() {
    empty_test(new RingBuffer());
  }
  
  private void empty_test(RingBuffer ring){
    assertEquals("empty ring count", 0, ring.getCount()); 
    try {
      ring.getFirst();
      fail("ring.getFirst(): Should have thrown exception.");
    }
    catch(IllegalStateException e) {
    }
    catch(Exception e) {
      fail("ring.getFirst(): Wrong exception: "+e);
    }

    try {
      ring.setFirst("x");
      fail("ring.setFirst(): Should have thrown exception.");
    }
    catch(IllegalStateException e) {
    }
    catch(Exception e) {
      fail("ring.setFirst(): Wrong exception: "+e);
    }
    
    try {
      ring.removeFront();
      fail("ring.removeFront(): Should have thrown exception.");
    }
    catch(IllegalStateException e) {
    }
    catch(Exception e) {
      fail("ring.removeFront(): Wrong exception: "+e);
    }

    ring.toNext(); // Should still be in empty state
    try {
      ring.getFirst();  // Test to see if still in empty state
      fail("ring.toNext(): Should have thrown exception.");
    }
    catch(IllegalStateException e) {
    }
    catch(Exception e) {
      fail("ring.toNext(): Wrong exception: "+e);
    }
    
    ring.toPrev(); // Should still be in empty state
    try {
      ring.getFirst();  // Test to see if still in empty state
      fail("ring.toPrev(): Should have thrown exception.");
    }
    catch(IllegalStateException e) {
    }
    catch(Exception e) {
      fail("ring.toPrev(): Wrong exception: "+e);
    }
  }
  
  public void test_one_element(){
    RingBuffer ring = new RingBuffer();
    assertEquals("ring.insertFront(42)", new Integer(42), (Integer)(ring.insertFront(42).getFirst()));
    one_element_test(ring);
  }
  
  private void one_element_test(RingBuffer ring){  
    assertEquals("ring.getCount()", 1, ring.getCount());
    assertEquals("ring.toNext().getFirst()", new Integer(42), (Integer)(ring.toNext().getFirst()));
    assertEquals("ring.toPrev().getFirst()", new Integer(42), (Integer)(ring.toPrev().getFirst()));

    assertEquals("ring.removeFront()", new Integer(42), (Integer)(ring.removeFront()));
    empty_test(ring);

  }
  
  
  public void test_many_elements() {
    RingBuffer ring = new RingBuffer();
    ring.insertFront(42).insertFront(99);
    assertEquals("99->42 ring", 2, ring.getCount());
    assertEquals("99->42 ring", 99, ring.getFirst());
    
    assertEquals("99->42 ring", 42, ring.toNext().getFirst());
    assertEquals("99->42 ring", 99, ring.toNext().getFirst());
    assertEquals("99->42 ring", 42, ring.toPrev().getFirst());
    assertEquals("99->42 ring", 99, ring.toPrev().getFirst());
    
    assertEquals("99->42 ring", 99, ring.removeFront());
  
    one_element_test(ring);
    
    ring = new RingBuffer();
    ring.insertFront(42).insertFront(99).insertFront(66);
    
    assertEquals("66->99->42 ring", 3, ring.getCount());
    assertEquals("66->99->42 ring", 66, ring.getFirst());
    
    assertEquals("66->99->42 ring", 99, ring.toNext().getFirst());
    assertEquals("66->99->42 ring", 42, ring.toNext().getFirst());
    assertEquals("66->99->42 ring", 66, ring.toNext().getFirst());
    assertEquals("66->99->42 ring", 42, ring.toPrev().getFirst());
    assertEquals("66->99->42 ring", 99, ring.toPrev().getFirst());
    
    assertEquals("99->42->66 ring", 99, ring.removeFront());
    assertEquals("42->66 ring", 42, ring.getFirst());
    assertEquals("42->66 ring", 66, ring.toNext().getFirst());
    assertEquals("42->66 ring", 42, ring.toNext().getFirst());
    assertEquals("42->66 ring", 66, ring.toPrev().getFirst());
    assertEquals("42->66 ring", 66, ring.removeFront());
    
    one_element_test(ring);
  }
}
